{{
Galois LFSR of up to 32 bit length
(C) 2009 virtuPIC@airspace-v.com

Galois LFSR generate periodic binary sequences. They can be used as
pseudo random generators (PRNG). The generated sequences have certain
properties.

You might consider LFSR as logic devices. They can be implemented in
software and - even worse - they are algebraic structures: The feedback
taps correspond to the coefficients of a polynomial.

We don't really need to care about that if we only want to generate an
arbitrary sequence or look at one of the many tables on the net to get a
polynomial - read 'tap sequence' - that gives maximum period. Maximum
period of an LFSR of n bit length is 2^n - 1. It will take all possible
contents exept all bits 0.  }}

CON
  'fb_polynomial is the feedback polynomial
  'a few samples for maximum period are given below.
  'the longer the SR the more maximum feedback polynomials exist
  'producing different sequences
  'find them by searching the net for: lfsr max period

  fb_polygon = $d0000001	'32 bit (untested)
  'fb_polygon = $0000d008	' 16 bit (untested)
  'fb_polygon = $000000b8	'  8 bit
  'fb_polygon = $0000000c	'  4 bit

  'seed to initialize SR
  'value must be different from 0
  'value should fit into shift register 
  default_seed = $db873f6a

VAR
  long state	'SR content

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'default seed the SR
PUB start
  if default_seed == 0
    abort	'notify caller of illegal value
  state := default_seed

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'seed SR
'for conditions on value look at default_seed
PUB seed(x)
  if x == 0
    abort	'notify caller of illegal value
  state := x

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'shift SR
'do lazy init
PUB nxt
  if state == 0
    if default_seed == 0
      abort
    state := default_seed
    return 0
  else
    state := (state >> 1) ^ (-(state & 1) & fb_polygon)
    return state

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
{{
Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
}}
